/** Basic DNA sequence
 */
#include <cstdlib>
#include <fstream>
#include <ios>
#include <iostream>
#include <type_traits>
#include <unistd.h>

#include "../include/Sequence.hpp"
#include "../include/GeneticCode.hpp"
#include "../include/fasta.hpp"

int main(int argc, char *argv[]) {
    std::string codon_filename = "resources/codons.txt";
    std::string fasta_filename = "";
    int opt;
    while ((opt = getopt (argc, argv, "i:c:h")) != -1) {
        switch(opt) {
            case 'i':
                fasta_filename = optarg;
                break;
            case 'c':
                codon_filename = optarg;
                break;
            case '?':
            case 'h':
            default:
                std::cout << "Usage: " << argv[0] << "-i <dna-fasta>" << std::endl;
                break;
            case -1:
                break;
        }
    }
    if (fasta_filename.length() == 0) {
        std::cerr << "Error: no fasta file given" << std::endl;
        return EXIT_FAILURE;
    }

    GeneticCode::GetInstance(codon_filename);
    std::ifstream fasta_ifstream(fasta_filename);
    std::vector<Record> fasta_records = fasta::read_file(fasta_ifstream);
    Record fasta_record(fasta_records.at(0));
    std::string sequence(fasta_record.seq);
    uppercase_string(sequence);
    DNASequence dna_sequence(sequence);
    RNASequence rna_sequence = dna_sequence.transcribe_to_rna();
    ProteinSequence protein_sequence = rna_sequence.translate_to_protein();
    std::cout << dna_sequence.get_sequence() << std::endl;
    std::cout << rna_sequence.get_sequence() << std::endl;
    std::cout << protein_sequence.get_sequence() << std::endl; 
    return EXIT_SUCCESS;
}